import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0

import "MapApi.js" as MapApi

Item {
    id: settingsBar
    anchors.fill: parent

    property variant mapView
    property int myBtnSize: 52

    Item {
        id: internals
        objectName: "mapSettingsInternals"

        property variant allowedStates: [
            [true, true, false],
            [true, true, true],
            [true, false, false],
            [true, false, true],
            [true, true, true]
        ]
        property int mapsColumn: 0
        property int satelliteColumn: 1
        property int terrainColumn: 2
        property int publicTransportRow: 0
        property int d3Row: 1
        property int nightmodeRow: 2
        property int landmarksRow: 3
        property int trafficRow: 4
        property bool mapInteractivityInitialValue: false
        property variant latestBehaviorChangeKey: null

        function changeMap(mode, pureApply) {
            if (mode >= 0 && mode <= 2) {
                mapTab.checked = false;
                satelliteTab.checked = false;
                terrainTab.checked = false;
                if (mode == mapsColumn) {
                    mapTab.checked = true;
                } else if (mode == satelliteColumn) {
                    satelliteTab.checked = true;
                } else if (mode == terrainColumn) {
                    terrainTab.checked = true;
                }

                publicTransport.active = allowedStates[publicTransportRow][mode];
                if (mapView.getBehavior(mapView._NO_PUBLIC_TRANSPORT_MODE_)) {
                    publicTransport.opacity = 0.5;
                } else {
                    publicTransport.opacity = 1.0;
                }

                d3.active = allowedStates[d3Row][mode];
                nightmode.active = allowedStates[nightmodeRow][mode];
                landmarks.active = allowedStates[landmarksRow][mode];
                traffic.active = allowedStates[trafficRow][mode];

                if (!pureApply) {
                    if (mode == mapsColumn) {
                        MapApi.MapLayers.setMapMode();
                    } else if (mode == satelliteColumn) {
                        MapApi.MapLayers.setSatelliteMode();
                    } else if (mode == terrainColumn) {
                        MapApi.MapLayers.setTerrainMode();
                    }
                }
            }
        }

    }

    Traffic {
        id: trafficObject
    }

    Timer {
        id: trafficTimer
        repeat: false
        interval: 500
        onTriggered: {
	    //trafficInfo.text="Triggered";
            trafficObject.requestTrafficAt(mapView.center, 50);
        }

    }

    Connections {
        target: mapView
        ignoreUnknownSignals: true

        onCenterChanged: { MapApi.MapLayers.mapViewCenterChanged() }
    }

    Component {
        id: spareGeoCoordinates

        GeoCoordinates { }
    }

    Item {
        id: settingsButton
        objectName: "settingsButton"
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        width:  74
        height: 74
        visible: true
        clip: true

        Image {
            id: image
            visible: true
            source: mapAssetsPath + "images/layout/settingsTab.png"
            states: State {
                name: "pressed"
                PropertyChanges { target: image; y: -70; }
            }
        }

        MouseArea {
            anchors.fill: settingsButton
            onPressed: image.state = "pressed"
            onClicked: {
                settingsBar.state = "open";

                publicTransport.checked = MapApi.MapLayers.isPublicTransportMode();
                d3.checked = MapApi.MapLayers.is3dMode();
                nightmode.checked = MapApi.MapLayers.isNightMode();
                landmarks.checked = MapApi.MapLayers.isLandmarksMode();
                traffic.checked = MapApi.MapLayers.isTrafficMode();

                internals.mapInteractivityInitialValue = mapView.getBehavior(mapView._NO_INTERACTIVITY_);
                if (!internals.mapInteractivityInitialValue) {
                    internals.latestBehaviorChangeKey = mapView.setBehavior(mapView._NO_INTERACTIVITY_, true);
                    mapView.blockInteractivity = true;
                }
            }
            onReleased: image.state = ""
        }
    }

    Rectangle {
        id: settingsMenu
        visible: false
        anchors.fill: parent
        color: "black"
        opacity: 0.9
        parent: mainscreen

        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }

        Rectangle {
            color: "transparent"

            anchors.left: parent.left
            anchors.right: content.left
            anchors.top: parent.top
            anchors.bottom: parent.bottom

            MouseArea {
                anchors.fill: parent
                onClicked: { settingsBar.state = "" }
            }
        }
        Rectangle {
            color: "transparent"

            anchors.left: content.right
            anchors.right: parent.right
            anchors.top: parent.top
            anchors.bottom: parent.bottom

            MouseArea {
                anchors.fill: parent
                onClicked: { settingsBar.state = "" }
            }
        }
        Rectangle {
            color: "transparent"

            anchors.left: content.left
            anchors.right: parent.right
            anchors.top: parent.top
            anchors.bottom: content.top

            MouseArea {
                anchors.fill: parent
                onClicked: { settingsBar.state = "" }
            }
        }
        Rectangle {
            color: "transparent"

            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top: content.bottom
            anchors.bottom: parent.bottom

            MouseArea {
                anchors.fill: parent
                onClicked: { settingsBar.state = "" }
            }
        }

        Column {
            id: content
            spacing: 16
            anchors {
                verticalCenter: parent.verticalCenter
                left: parent.left
                right: parent.right
                leftMargin: 20
                rightMargin: 20
            }

            Item {
                width: parent.width
                height: 20

                Text {
                    objectName: "mapOptionHeadline"
                    text: qsTranslate("Maps", "qtn_mapconnector_settings_mapoption"); //"Map Options"
                    color: "#1080dd"
                    font.pixelSize: 25
                    font.bold: true

                    anchors.verticalCenter: parent.verticalCenter
                }

                Image {
                    source: mapAssetsPath + "images/layout/close.png"
                    anchors.right: parent.right

                    anchors.verticalCenter: parent.verticalCenter

                    MouseArea {
                        anchors.fill: parent
                        onClicked: { settingsBar.state = "" }
                    }
                }
            }

            // Line
            Rectangle { color: "#1080dd"; height: 1; width: parent.width  }

            ButtonRow {
                width: parent.width
                height: 50
                platformStyle: ButtonStyle { inverted: true;}
                Button {
                    id: mapTab
                    objectName: "mapTabButton"
                    text:  qsTranslate("Maps", "qtn_mapconnector_settings_map"); //"Map"
                    checked: MapApi.MapLayers.isMapMode()
                    onClicked: internals.changeMap(internals.mapsColumn, false)
                }
                Button {
                    id: satelliteTab
                    objectName: "satelliteTabButton"
                    text:  qsTranslate("Maps", "qtn_mapconnector_settings_sat"); //"Satellite"
                    checked: MapApi.MapLayers.isSatelliteMode()
                    onClicked: internals.changeMap(internals.satelliteColumn, false)
                }
                Button {
                    id: terrainTab
                    objectName: "terrainTabButton"
                    text:  qsTranslate("Maps", "qtn_mapconnector_settings_terrain"); //"Terrain"
                    checked: MapApi.MapLayers.isTerrainMode()
                    onClicked: internals.changeMap(internals.terrainColumn, false)
                }
            }

            // Line
            Rectangle { color: "#FFFFFF"; height: 1; width: parent.width  }

            // Option switchers
            Column {
                spacing: 10
                width: parent.width

                SettingOption {
                    id: publicTransport
                    objectName: "publicTransport"

                    text:  qsTranslate("Maps", "qtn_mapconnector_settings_pt"); //"Public transport"

                    checked: MapApi.MapLayers.isPublicTransportMode()
                    onStateChanged: { if (publicTransport.opacity == 1) { MapApi.MapLayers.changePublicTransportMode(); } else { publicTransport.checked = !publicTransport.checked; } }
                }
                SettingOption {
                    id: d3
                    objectName: "d3"

                    text:  qsTranslate("Maps", "qtn_mapconnector_settings_3d"); //"3D"

                    checked: MapApi.MapLayers.is3dMode()
                    onStateChanged: { MapApi.MapLayers.change2d3dMode() }
                }
                SettingOption {
                    id: nightmode
                    objectName: "nightmode"

                    text:  qsTranslate("Maps", "qtn_mapconnector_settings_nightmode"); //"Nightmode"

                    checked: MapApi.MapLayers.isNightMode()
                    onStateChanged: { MapApi.MapLayers.changeDayNightMode() }
                }
                SettingOption {
                    id: landmarks
                    objectName: "landmarks"

                    text:  qsTranslate("Maps", "qtn_mapconnector_settings_landmarks"); //"Landmarks"

                    checked: MapApi.MapLayers.isLandmarksMode()
                    onStateChanged: { MapApi.MapLayers.changeLandmarksMode() }
                }
                SettingOption {
                    id: traffic
                    objectName: "traffic"
                    visible: true // ak20110329 - requested override

                    text:  qsTranslate("Maps", "qtn_mapconnector_settings_traffic"); //"Traffic"

                    checked: MapApi.MapLayers.isTrafficMode()
                    onStateChanged: { MapApi.MapLayers.changeTrafficMode() }
                }
            }

            // Line
            Rectangle { color: "#1080dd"; height: 1; width: parent.width  }

            onVisibleChanged:  {
                if (visible) {
                    if (MapApi.MapLayers.isMapMode()) {
                        internals.changeMap(internals.mapsColumn, true);
                    } else if (MapApi.MapLayers.isSatelliteMode()) {
                        internals.changeMap(internals.satelliteColumn, true);
                    } else if (MapApi.MapLayers.isTerrainMode()) {
                        internals.changeMap(internals.terrainColumn, true);
                    }
                }
            }
        }
    }

    states:  State {
        name: "open"
        PropertyChanges { target: settingsButton; visible: false }
        PropertyChanges { target: settingsMenu;   visible: true }
    }

    onStateChanged: {
        if (!state) {
            if (!internals.mapInteractivityInitialValue) {
                if (mapView.getBehavior(mapView._NO_INTERACTIVITY_) && mapView.getCurrentBehaviorStateRandomCode() == internals.latestBehaviorChangeKey) {
                    mapView.setBehavior(mapView._NO_INTERACTIVITY_, false);
                    mapView.blockInteractivity = false;
                }
            }
        }
    }
}
