import QtQuick 1.0
import MapsPlugin 1.0

import "MapViewer.js" as EventHandlers
import "MapApi.js" as MapApi

Map {
    id: mapView

    property string pluginVersion: MapsPlugin.pluginVersion
    property string mapVersion: MapsPlugin.mapVersion
    property string mosCleintId: MapsPlugin.clientId
    property string mapConnectorVersion: appState.mapsVersion

    property int _NO_MAINSCREEN_: -1
    property variant mainScreen

    deviceOrientationMode: EventHandlers.setDeviceOrientation()

    property variant mapSharedData: MapSharedData {}
    property variant mapGlobalData: MapGlobalData {}

    property bool kineticsEnabled: true
    property int kineticsHalfLife: 125
    property variant kineticsHalfLifes: [0, 35, 35, 35, 35, 35, 35, 35, 65, 65, 65, 95, 95, 125, 125, 145, 165, 165, 165]

    property variant poiCategoryFilter: [
        Icon.Airport, Icon.AirLineAccess, Icon.RailwayStation, Icon.RailwayAccess, Icon.FerryTerminal,
        Icon.MetroAccess, Icon.MetroStation
    ]

    property bool ready: false

    signal attachedToLocationChanged()
    property alias attachedToLocation: myPositionBar.attachedToLocation
    property alias skipAnimationOnAttachedToLocation: myPositionBar.skipAnimationOnAttachedToLocation

    property alias positionProviderEnabled: myPositionBar.positionProviderEnabled

    signal currentPositionChanged()
    signal previousPositionChanged()
    signal positionTypeChanged()
    signal drawnHaloSizeChanged()
    property alias currentPosition: myPositionBar.currentPosition
    property alias previousPosition: myPositionBar.previousPosition
    property alias positionType: myPositionBar.positionType
    property alias drawnHaloSize: myPositionBar.drawnHaloSize
    property alias averageSpeed: myPositionBar.averageSpeed
    property alias compassMode: myPositionBar.compassMode
    property alias previousCompassMode: myPositionBar.previousCompassMode
    property alias referenceToMapMatchedPosition: myPositionBar.referenceToMapMatchedPosition
    property alias referenceToDevicePosition: myPositionBar.referenceToDevicePosition
    property string _POSITION_TYPE_NONE_: "_POSITION_TYPE_NONE_"
    property string _POSITION_TYPE_GPS_: "_POSITION_TYPE_GPS_"
    property string _POSITION_TYPE_WIFI_CELL_: "_POSITION_TYPE_WIFI_CELL_"

    property bool preventAutomaticPropertyFiltering: true

    property int farDragPixelDistance: 50;

    property int _DEFAULT_MODE_: 0
    property int _NO_POSITION_AT_ALL_: 1
    property int mode: mapView._DEFAULT_MODE_

    property int _DEFAULT_STATUS_: 0
    property int _INTERRUPTED_ANIMATION_: 1
    property int status: mapView._DEFAULT_STATUS_

    property bool invisibleBlockInteractivity: false
    property bool blockInteractivity: false
    property string _POSITION_TYPE_REQUIRED_GPS_: "_POSITION_TYPE_REQUIRED_GPS_"
    property string _NO_INTERACTIVITY_: "_NO_INTERACTIVITY_"
    property string _NO_SETTINGS_: "_NO_SETTINGS_"
    property string _NO_AUTO_GPS_INIT_: "_NO_AUTO_GPS_INIT_"
    property string _NO_POSITION_MARKER_: "_NO_POSITION_MARKER_"
    property string _NO_LONG_PRESS_AT_ALL_: "_NO_LONG_PRESS_AT_ALL_"
    property string _NO_PUBLIC_TRANSPORT_STOPS_CLICK_: "_NO_PUBLIC_TRANSPORT_STOPS_CLICK_"
    property string _NO_PUBLIC_TRANSPORT_MODE_: "_NO_PUBLIC_TRANSPORT_MODE_"
    property string _NO_INFO_BUBBLE_DEFAULT_LONG_PRESS_ACTION_: "_NO_INFO_BUBBLE_DEFAULT_LONG_PRESS_ACTION_"
    property string _NO_INFO_BUBBLE_DEFAULT_OBJECT_PRESS_ACTION_: "_NO_INFO_BUBBLE_DEFAULT_OBJECT_PRESS_ACTION_"
    property string _NO_INFO_BUBBLE_CLICK_EFFECT_: "_NO_INFO_BUBBLE_CLICK_EFFECT_"
    property string _NO_MY_POSITION_DEFAULT_CLICK_ACTION_: "_NO_MY_POSITION_DEFAULT_CLICK_ACTION_"
    property string _NO_MY_POSITION_CLICK_ACTION_AT_ALL_: "_NO_MY_POSITION_CLICK_ACTION_AT_ALL_"
    property string _NO_COMPASS_ALLOWED_: "_NO_COMPASS_ALLOWED_"
    property string _NO_INITIAL_SETTINGS_LOAD_: "_NO_INITIAL_SETTINGS_LOAD_"
    signal behaviorsModified(string key, bool value)
    property variant __behaviors
    property variant __currentBehaviorStateRandomCode: "..."
    function getCurrentBehaviorStateRandomCode() { return mapView.__currentBehaviorStateRandomCode; }
    function setBehavior(key, value) { var tmpBehaviors = mapView.__behaviors || {}; tmpBehaviors[key] = value; mapView.__behaviors = tmpBehaviors; mapView.behaviorsModified(key, value === "true"); mapView.__currentBehaviorStateRandomCode = Math.random(); return mapView.__currentBehaviorStateRandomCode; }
    function getBehavior(key) { var tmpBehaviors = mapView.__behaviors; return tmpBehaviors && tmpBehaviors[key] === "true" || tmpBehaviors[key] === true; }
    property string behaviors: "_POSITION_TYPE_REQUIRED_GPS_:false;_NO_AUTO_GPS_INIT_:false;_NO_POSITION_MARKER_:false;_NO_LONG_PRESS_AT_ALL_:false;_NO_PUBLIC_TRANSPORT_STOPS_CLICK_:false;_NO_PUBLIC_TRANSPORT_MODE_:false;_NO_INTERACTIVITY_:false;_NO_SETTINGS_:false;_NO_INFO_BUBBLE_DEFAULT_LONG_PRESS_ACTION_:false;_NO_INFO_BUBBLE_DEFAULT_OBJECT_PRESS_ACTION_:false;_NO_INFO_BUBBLE_CLICK_EFFECT_:false;_NO_MY_POSITION_DEFAULT_CLICK_ACTION_:false;_NO_MY_POSITION_CLICK_ACTION_AT_ALL_:false;_NO_COMPASS_ALLOWED_:false;_NO_INITIAL_SETTINGS_LOAD_:false;"
    onBehaviorsChanged: { EventHandlers.setBehaviors(mapView.behaviors) }

    property double startupCenterLatitude
    property double startupCenterLongitude
    property double startupZoomLevel

    onPoiCategoryFilterChanged: { MapApi.applyPoiCategoryFilter(poiCategoryFilter) }

    property bool dontSaveData: false
    property bool dontPopulateGlobalActiveMap: false

    /**
      * Acceptable values (for now)
      *
      * (o) close-infobubble - Closes the opened infobubble
      * (o) force-close-infobubble - Force close infobubble
      * (o) clear-transit-hightlights - Clears all transit hightlights
      * (o) conditional-clear-all - Clears conditional all states
      * (o) emit-force-close-infobubble - Emits force-close-infobubble to all maps that share one mapSharedData
      * (o) ui-changed - Redraws the map visual settings
      * (o) emit-ui-changed - Emits ui-changed to all maps that share one mapSharedData
      * (o) forget-selected-marker - Tells the map to forget about it's selected marker
      * (o) emit-forget-selected-marker - Emits forget-selected-marker to all maps that share one mapSharedData
      */
    property variant control
    function setControl(control) {
        mapView.control = "__fake__";
        mapView.control = control;
    }
    property variant propaganda
    function propagandize(newAction, newPropaganda) {
        mapView.mapSharedData.propagandize(newAction, newPropaganda);
    }
    property variant stash

    property bool lazyAppendIconMode: false

    property bool useGlobalShare: false
    property variant model
    onModelChanged: { EventHandlers.onModelChanged(mapView.model) }
    property variant modelLayer
    property Component modelFactory: mainscreen.modelFactory
    property Component modelMarkerFactory: mainscreen.modelMarkerFactory
    property Component modelLayerFactory: mainscreen.modelLayerFactory

    property DebugOverlayPanel __debugLogger: debugLogger

    signal mapCurrentLocationClicked(variant pixelPos, variant geoPos, variant mapObject)
    signal mapObjectsSelected(variant mapObject)
    signal mapClicked(variant pixelPos, variant geoPos, variant mapObject)
    signal mapDoubleClicked(variant pixelPos, variant geoPos)
    signal mapTwoFingerDoubleClicked(variant pixelPos, variant geoPos)
    signal twoFingerSlide(int deltaX, int deltaY)
    signal mapDragStarted(variant pixelPos, variant geoPos)
    signal mapDragged(variant pixelPos, variant geoPos)
    signal mapDragEnded(variant pixelPos, variant geoPos)
    signal mapDraggedFar(variant pixelPos, variant geoPos)
    signal mapLongPressed(variant pixelPos, variant geoPos)
    signal mapLongPressRectangleCompleted(variant pixelPos, variant geoPos)
    signal mapLongPressRectangleCanceled(variant pixelPos, variant geoPos)
    signal mapReleased(variant pixelPos, variant geoPos)
    signal placeSelected(string uri)
    signal positionSelected(variant positionData, bool currentPositionSelected)
    signal flyToCurrentPosition()
    signal myPositionBarClicked()

    function startPositionProvider() { myPositionBar.startPositionProvider() }
    function stopPositionProvider() { myPositionBar.stopPositionProvider() }
    function disablePositionProvider() { myPositionBar.disablePositionProvider() }
    function enablePositionProvider() { myPositionBar.enablePositionProvider() }
    function forceChangeAccuracy(accuracy) { myPositionBar.forceChangeAccuracy(accuracy) }

    function callMapApi(method, value) {
        return MapApi[method].apply(null, value);
    }

    function callMapLayersApi(method, value) {
        MapApi.MapLayers.init();
        return MapApi.MapLayers[method].apply(null, value);
    }

    function doFlyToCurrentPosition() {
        myPositionBar.flyToCurrentPosition();
    }

    function showRightPopupMessage(text) {
        myPositionBar.__rightPopup.message = text;
        myPositionBar.__rightPopup.hideTimeMultiplier = 99999;
        myPositionBar.__rightPopup.dontClosePopupOnMouseMove = true;
    }
    function showInfoBubble(marker) {
        if (infoBubbleBehavior) {
            infoBubbleBehavior.showMarker(marker);
            mapView.model.selectObject(marker);
        }
    }
    function showInfoBubbleWithContent(content) {
        if (infoBubbleBehavior) {
            infoBubbleBehavior.showContent(content);
        }
    }
    function showInfoBubbleWithContentAt(geoCoordinates, content) {
        if (infoBubbleBehavior) {
            infoBubbleBehavior.showAt(geoCoordinates, content);
        }
    }
    function showInfoBubbleFromMap(geoCoordinates, extraData) {
        if (infoBubbleBehavior) {
            infoBubbleBehavior.showFromMap(geoCoordinates, extraData);
        }
    }

    function closeInfoBubble(animated) {
        if (infoBubbleBehavior) {
            infoBubbleBehavior.resetAll(!animated);
        }
    }
    function closeInfoBubbleByForce() {
        if (infoBubbleBehavior) {
            infoBubbleBehavior.forceResetAll();
        }
    }

    function getListView(data) {
        return infoBubbleBehavior.getListView(data);
    }

    function getMapBoundingBox(x, y, width, height, angle) {
        return EventHandlers.handleGetBoundingBox(x, y, width, height, angle);
    }

    qmlItems: [
        MapQmlItem {
            center: mapView.referenceToMapMatchedPosition? mapView.referenceToMapMatchedPosition.geoCoordinates : null

            QmlItemPositionMarker {
                accuracy: mapView.referenceToMapMatchedPosition? mapView.referenceToMapMatchedPosition.accuracy / mapView.zoomScale * 100 : 1
                itemColor: "#ff0000"
            }
        },
        MapQmlItem {
            center: mapView.referenceToDevicePosition? mapView.referenceToDevicePosition.geoCoordinates : null

            QmlItemPositionMarker {
                accuracy: mapView.referenceToDevicePosition? mapView.referenceToDevicePosition.accuracy / mapView.zoomScale * 100 : 1
                itemColor: "#00ff00"
            }
        }
    ]

    children: [
        /*
        Rectangle {
            width: 100
            height: 100
            color: "red"

            Component {
                id: tttt

                Rectangle {
                    visible: false

                    width: 600
                    height: 100
                    color: "pink"

                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            log.info(".")
                        }
                    }
                }
            }

            Text {
                id: ttt
                anchors.fill: parent
            }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    //mapView.perspective += 1;
                    //mapView.landmarksVisible = true;
                    //mapView.trafficInfoVisible = true;
                    //mapView.transitMode = Map.TRANSIT_MODE_SHOW_EVERYTHING;
                    //mapView.transitMode = Map.TRANSIT_MODE_SHOW_STOPS_AND_ACCESSES;
                    //mapView.scheme = "pedestrian.day";
                    //mapView.scheme = "satellite.night";
                    //mapView.scheme = "terrain.night";
                    //mapView.scheme = "normal.day.transit"
                    //mapView.scheme = "normal.night.transit";
                    //mapView.scheme = "hybrid.day.transit";
                    //mapView.scheme = "hybrid.night.transit";

                    //MapApi.zoomToLevel(18);
                    //mapView.kineticsEnabled = !mapView.kineticsEnabled;

                    //mapView.showInfoBubbleWithContentAt(mapView.currentPosition.geoCoordinates, tttt.createObject(mapView));

                    //mapView.showRightPopupMessage("hahahahahaha");

                    //log.info(mapView.callMapLayersApi("setPublicTransportModeOn", null))
                    //log.info(mapView.callMapLayersApi("setPublicTransportModePartialOn", null))

                    //ttt.text = qsTranslate("Maps", "qtn_maps_countries_AFG")

                    console.log("................ " + appState.getSystemTmpDir())
                    mapView.saveScreenshot("/tmp/screen0.png");
                    console.log("................ " + appState.getSystemTmpDir())

                    console.log("............     " + mapView.scheme)
                }
            }
        },
        */

        Image {
            anchors.bottom: parent.bottom
            anchors.left: myPositionBar.right
            anchors.leftMargin: myPositionBar.myBtnSize + 10
            anchors.bottomMargin: 10
            source: mapAssetsPath + "images/layout/Nokia-brand-white.png"
            visible: !mapView.blockInteractivity && !mapView.getBehavior(mapView._NO_INTERACTIVITY_)
        },

        MapEventHandler {
            mapView: mapView
            longPressRectangle: longPressRectangle
        },

        MapBehaviors {
            mapView: mapView
        },

        InfoBubbleBehavior {
            id: infoBubbleBehavior
            mapView: mapView
            ui: infoBubble
        },

        ZoomBar {
            id: zoomBar
            mapView: mapView
            anchors {
                bottom: parent.bottom
                horizontalCenter: parent.horizontalCenter
            }

            visible: {
                return !mapView.blockInteractivity && !mapView.getBehavior(mapView._NO_INTERACTIVITY_) && (!appState.isDevice || appState.zoombarVisible || mainscreen.zoombarsEnabled)
            }

            opacity: myPositionBar.__rightPopup.visible? 0 : 1
        },

        MyPositionBar {
            id: myPositionBar
            mapView: mapView
            anchors.left: parent.left
            anchors.bottom: parent.bottom
            anchors.leftMargin: 10
            anchors.bottomMargin: 10

            onAttachedToLocationChanged: { mapView.attachedToLocationChanged() }
            onCurrentPositionChanged: { mapView.currentPositionChanged() }
            onPreviousPositionChanged: { mapView.previousPositionChanged() }
            onPositionTypeChanged: { mapView.positionTypeChanged() }
            onDrawnHaloSizeChanged: { mapView.drawnHaloSizeChanged() }

            visible: !mapView.blockInteractivity && !mapView.getBehavior(mapView._NO_INTERACTIVITY_)
        },

        LongPressRectangle {
            id: longPressRectangle
            mapView: mapView
        },

        MapSettingsUI {
            mapView: mapView

            visible: !mapView.getBehavior(mapView._NO_SETTINGS_) && !mapView.blockInteractivity && !mapView.getBehavior(mapView._NO_INTERACTIVITY_)
        },

        DotBlinker {
            id: greenDot

            color: "green"

            anchors.top: parent.top
            anchors.right: parent.right
            anchors.topMargin: 10
            anchors.rightMargin: 10

            Connections {
                target: myPositionBar
                ignoreUnknownSignals: true

                onPositionUpdatedWithValidPosition: {
                    greenDot.blink();
                }
            }
        },
        DotBlinker {
            color: "red"

            numberOfBlinks: 3

            anchors.top: parent.top
            anchors.right: parent.right
            anchors.topMargin: 10
            anchors.rightMargin: 10 + 30 + 10

            Connections {
                target: myPositionBar
                ignoreUnknownSignals: true

                onPositionUpdatedWithInvalidPosition: {
                    greenDot.blink();
                }
            }
        },
        DotBlinker {
            color: "yellow"

            numberOfBlinks: 5

            anchors.top: parent.top
            anchors.right: parent.right
            anchors.topMargin: 10
            anchors.rightMargin: 10 + 30 + 10 + 30 + 10

            Connections {
                target: myPositionBar
                ignoreUnknownSignals: true

                onPositionLost: {
                    greenDot.blink();
                }
            }
        },

        Item {
            anchors.fill: parent
            MouseArea {
                anchors.fill: parent
                onClicked: {}
            }
            visible: !mapView.ready || mapView.blockInteractivity || mapView.getBehavior(mapView._NO_INTERACTIVITY_)
        },

        InfoBubbleUI {
            id: infoBubble
            anchors.centerIn: mapView

            mapView: mapView
        },

         DebugOverlayPanel {
             id: debugLogger
         }
        // TMO patch
	//Text { id: trafficInfo; font.bold: true; font.pixelSize:24; text: "Loading Traffic"}
     ]

    Component.onCompleted: {
        log.info("\n\n\nNEW MAP : " + mapView + "\n\n\n");
        EventHandlers.handleOnLoad();
    }
    Component.onDestruction: {
        log.info("\n\n\nMAP DESTROYED : " + mapView + "\n\n\n");
    }
}
